package com.mcsnowflake.worktimer.ui

import android.app.NotificationManager
import android.media.AudioManager
import android.media.AudioManager.RINGER_MODE_VIBRATE
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.AlertDialog
import androidx.compose.material.Button
import androidx.compose.material.Icon
import androidx.compose.material.IconButton
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Surface
import androidx.compose.material.Text
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.rounded.Settings
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.State
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavHostController
import com.mcsnowflake.worktimer.PREFERENCES_SCREEN
import com.mcsnowflake.worktimer.ui.progressView.ProgressView
import com.mcsnowflake.worktimer.ui.setupView.SetupView
import com.mcsnowflake.worktimer.ui.shared.AppBar
import com.mcsnowflake.worktimer.ui.shared.AppBarStatus
import org.koin.androidx.compose.get
import org.koin.androidx.compose.getViewModel

const val DASHBOARD = "dashboard"

@Composable
fun Dashboard(
    navController: NavHostController,
    modifier: Modifier = Modifier,
    audioManager: AudioManager = get(),
    dashBoardViewModel: DashBoardViewModel = getViewModel()
) = Column(modifier.fillMaxSize()) {
    val showMuteWarning = remember { mutableStateOf(audioManager.ringerMode != AudioManager.RINGER_MODE_NORMAL) }
    AppBar(
        navController,
        AppBarStatus(
            title = "WorkTimer",
            actionIcons = { IconButton({ it.navigate(PREFERENCES_SCREEN) }) { Icon(Icons.Rounded.Settings, "Settings") } },
            showMenu = true
        )
    )
    Surface(
        Modifier.fillMaxSize(),
        color = MaterialTheme.colors.background
    ) {
        Column(
            Modifier
                .fillMaxSize()
                .padding(32.dp)
        ) {
            Header(Modifier.weight(3F), dashBoardViewModel.headerText.collectAsState())
            Box(Modifier.weight(10F)) {
                SetupView(Modifier.fillMaxSize(), dashBoardViewModel.isNotRunning.collectAsState())
                ProgressView(Modifier.fillMaxSize(), dashBoardViewModel)
            }
        }
    }
    MuteWarning(showMuteWarning)
}

@Composable
fun Header(modifier: Modifier = Modifier, headerText: State<String>) =
    Box(modifier.fillMaxWidth(), Alignment.Center) { Text(headerText.value, fontSize = 30.sp) }

@Composable
fun MuteWarning(
    showDialog: MutableState<Boolean>,
    notificationManager: NotificationManager = get(),
    audioManager: AudioManager = get()
) {
    if (showDialog.value) {
        AlertDialog(
            onDismissRequest = {
                showDialog.value = false
            },
            title = {
                Text("Warning")
            },
            confirmButton = {
                Button(
                    onClick = { showDialog.value = false },
                ) {
                    Text("Acknowledged")
                }
            },
            dismissButton = {
                if (notificationManager.isNotificationPolicyAccessGranted) {
                    Button(
                        onClick = {
                            showDialog.value = false
                            audioManager.ringerMode = AudioManager.RINGER_MODE_NORMAL
                        },
                    ) {
                        Text("Set audio to normal")
                    }
                }
            },
            text = {
                Column(
                    verticalArrangement = Arrangement.spacedBy(8.dp)
                ) {
                    Text("Your phone is set to ${if (audioManager.ringerMode == RINGER_MODE_VIBRATE) "vibrate only" else "be silent"}.")
                    Text("Because of that you might miss alarms the WorkTimer is issuing.")
                }
            },
        )
    }
}
